indexing

	description: "Class that describes socket addresses, i.e. host name%
	%and port number combination. Or in C terms struct sockaddr_in."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


class

	SUS_SOCKET_ADDRESS


inherit

	SUS_BASE

	SAPI_IN


creation

	make


feature -- Creation

	make (a_host: SUS_HOST; a_service: SUS_SERVICE) is
			-- Initialize socket for known host.
		require
			valid_host: a_host /= Void and then a_host.found
			valid_service: a_service /= Void
		do
			host := a_host
			service := a_service
			-- ip4 only for now
			create buf.allocate_and_clear (posix_sockaddr_in_size)
			set_address (host.addresses.lower)
		ensure
			host_set: host = a_host
			service_set: service = a_service
		end


feature -- Public state

	host: SUS_HOST
			-- Resolved host name.

	service: SUS_SERVICE
			-- Port and protocol (udp/tcp) type.


feature -- Fill socket structure, so `ptr' returns something valid

	set_address (item: INTEGER) is
			-- Use one of the ip addresses of `host' as the socket address.
		require
			existing_ip_address_item:
				item >= host.addresses.lower and
				item <= host.addresses.upper
		local
			ip_address: SUS_IP_ADDRESS
		do
			posix_set_sockaddr_in_sin_family (buf.ptr, host.address_type)
			posix_set_sockaddr_in_sin_port (buf.ptr, posix_htons (service.port))
			ip_address := host.addresses.item (item)
			posix_set_sockaddr_in_sin_addr (buf.ptr, ip_address.ptr)
		end


feature -- Features the C API calls like

	length: INTEGER is
			-- Size of my struct sockaddr_in.
		do
			Result := buf.capacity
		ensure
			valid_length: length > 0 -- >= 16 usually
		end

	ptr: POINTER is
			-- Points to struct sockaddr_in or sockaddr_in6.
		do
			Result := buf.ptr
		ensure
			valid_result: Result /= default_pointer
		end


feature {NONE} -- Private state

	buf: STDC_BUFFER
			-- Pointer to a struct sockaddr_in structure.


invariant

	host_found: host /= Void and then host.found
	has_service: service /= Void
	valid_buf: buf /= Void and then buf.capacity >= length

end
